/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "common.h"

int _pools = 0;
int _poolblocks = 0;

pool pool_new()
{
    pool p;
    p = malloc(sizeof(_pool));
    p->size = 0;
    p->block = NULL;
    p->next = NULL;
    p->key = 0;
    ++_pools;
/*    jdebug("pool","%d +POOL %d %d",p,_pools,_poolblocks);*/
    return p;
}

void *pmalloc(pool p, int size)
{
    void *block = NULL;
    pool new;

    block = malloc(size);

    if(p == NULL) /* this is most likely an error that we got a NULL here! */
    {
        fprintf(stderr,"Memory Leak! [pmalloc received NULL pool, unable to track allocation]");
    }else{
        new = malloc(sizeof(_pool));
        new->size = size;
        _poolblocks += size;
        new->block = block;
        new->next = p->next;
        p->size += size;
        p->next = new;
    }

    return block;
}

void *pmalloc_key(pool p, int key, int size)
{
    void *block = NULL;
    pool next;

    next = p;
    while(next != NULL)
    {
        if(next->key == key)
            return next->block;
        next = next->next;
    }

    if(size != 0)
    {
        block = pmalloc(p, size);
        p->next->key = key;
    }

    return block;
}

void *pool_blockcheck(pool p, void *block)
{
    pool next;

    next = p;
    while(next != NULL)
    {
        if(next->block == block)
            return next->block;
        next = next->next;
    }

    return NULL;
}

char *pstrdup(pool p, const char *src)
{
    char *ret;

    if(src == NULL)
        return NULL;

/* duh... though I could erk out some savings by not duplicating strings already in the pool, NOT if the string get's disected/changed
    if(pool_blockcheck(p, (void *)src) == NULL)
    {
        ret = pmalloc(p,strlen(src) + 1);
        strcpy(ret,src);
    }else{
        ret = (char *)src;
    }
*/
    ret = pmalloc(p,strlen(src) + 1);
    strcpy(ret,src);

    return ret;
}

int pool_size(pool p)
{
    if(p == NULL) return 0;

    return p->size;
}

void pool_free(pool p)
{
    pool next, current;
    --_pools;

    p->size = 0;
    current = p;
    while(current != NULL)
    {
        next = current->next;
        _poolblocks -= current->size;
        free(current->block);
        free(current);
        current = next;
    }
/*    _jdebug("pool","%d -POOL %d %d",p,_pools,_poolblocks);*/
}
