/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "common.h"

char *j_strdup(const char *str)
{
	if(str == NULL)
		return NULL;
	else
		return strdup(str);
}

int j_strcmp(const char *a, const char *b)
{
	if(a == NULL || b == NULL)
		return -1;
	else
		return strcmp(a, b);
}

spool spool_new(pool p)
{
    spool s;

    s = pmalloc(p, sizeof(struct spool_struct));
    s->p = p;
    s->len = 0;
    s->last = NULL;
    s->first = NULL;
    return s;
}

void spool_add(spool s, char *str)
{
    struct spool_node *sn;
    int len;

    if(str == NULL)
        return;

    len = strlen(str);
    if(len == 0)
        return;

    sn = pmalloc(s->p, sizeof(struct spool_node));
    sn->c = pstrdup(s->p, str);
    sn->next = NULL;

    s->len += len;
    if(s->last != NULL)
        s->last->next = sn;
    s->last = sn;
    if(s->first == NULL)
        s->first = sn;
}

void spooler(spool s, ...)
{
    va_list ap;
    char *arg = NULL;

    if(s == NULL)
        return;

    va_start(ap, s);

    /* loop till we hit our end flag, the first arg */
    while(1)
    {
        arg = va_arg(ap,char *);
        if((int)arg == (int)s)
            break;
        else
            spool_add(s, arg);
    }

    va_end(ap);
}

char *spool_print(spool s)
{
    char *ret;
    struct spool_node *next;

    if(s == NULL || s->len == 0 || s->first == NULL)
        return NULL;

    ret = pmalloc(s->p, s->len + 1);
    *ret = '\0';

    next = s->first;
    while(next != NULL)
    {
        strcat(ret,next->c);
        next = next->next;
    }

    return ret;
}


char *strunescape(pool p, char *buf)
{
	int i,j=0;
	char *temp;

	if (p == NULL || buf == NULL) return(NULL);

        if (strchr(buf,'&') == NULL) return(buf);

	temp = pmalloc(p,strlen(buf)+1);

	if (temp == NULL) return(NULL);

	for(i=0;i<strlen(buf);i++)
        {
		if (buf[i]=='&')
                {
			if (strncmp(&buf[i],"&amp;",5)==0)
                        {
				temp[j] = '&';
				i += 4;
			} else if (strncmp(&buf[i],"&quot;",6)==0) {
				temp[j] = '\"';
				i += 5;
			} else if (strncmp(&buf[i],"&apos;",6)==0) {
				temp[j] = '\'';
				i += 5;
			} else if (strncmp(&buf[i],"&lt;",4)==0) {
				temp[j] = '<';
				i += 3;
			} else if (strncmp(&buf[i],"&gt;",4)==0) {
				temp[j] = '>';
				i += 3;
                        }
		} else {
			temp[j]=buf[i];
		}
		j++;
	}
	temp[j]='\0';
	return(temp);
}


char *strescape(pool p, char *buf)
{
	int i,j,oldlen,newlen;
	char *temp;

	if (p == NULL || buf == NULL) return(NULL);

	oldlen = newlen = strlen(buf);
	for(i=0;i<oldlen;i++)
        {
		switch(buf[i])
                {
			case '&':
				newlen+=5;
				break;
			case '\'':
				newlen+=6;
				break;
			case '\"':
				newlen+=6;
				break;
			case '<':
				newlen+=4;
				break;
			case '>':
				newlen+=4;
				break;
		}
	}

        if(oldlen == newlen) return buf;

	temp = pmalloc(p,newlen+1);

	if (temp==NULL) return(NULL);

	for(i=j=0;i<oldlen;i++)
        {
		switch(buf[i])
                {
			case '&':
				memcpy(&temp[j],"&amp;",5);
				j += 5;
				break;
			case '\'':
				memcpy(&temp[j],"&apos;",6);
				j += 6;
				break;
			case '\"':
				memcpy(&temp[j],"&quot;",6);
				j += 6;
				break;
			case '<':
				memcpy(&temp[j],"&lt;",4);
				j += 4;
				break;
			case '>':
				memcpy(&temp[j],"&gt;",4);
				j += 4;
				break;
			default:
				temp[j++] = buf[i];
		}
	}
	temp[j] = '\0';
	return temp;
}
