/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"


jid jid_new(pool p, char *idstr)
{
    char *server, *resource, *type, *str;
    jid id;

    if(p == NULL || idstr == NULL)
        return NULL;

    /* type:user@server/resource */

    str = pstrdup(p, idstr);

    id = pmalloc(p,sizeof(struct jid_struct));
    id->full = id->server = id->user = id->resource = id->type = NULL;
    id->p = p;
    id->next = NULL;

    resource = strstr(str,"/");
    if(resource != NULL)
    {
        *resource = '\0';
        ++resource;
        id->resource = resource;
    }
    
    type = strstr(str,":");
    if(type != NULL)
    {
        *type = '\0';
        ++type;
        if(strcmp(str,"jabber") != 0)
            id->type = str;
        str = type;
    }

    server = strstr(str,"@");
    if(server == NULL)
    { /* if there's no @, it's just the server address */
        id->server = str;
        return id;
    }else{
        *server = '\0';
        ++server;
        id->server = server;
        id->user = str;
    }

    return id;
}

void jid_set(jid id, char *str, int item)
{
    if(id == NULL)
        return;

    /* invalidate the cached copy */
    id->full = NULL;

    switch(item)
    {
        case JID_RESOURCE:
            id->resource = pstrdup(id->p, str);
            break;
        case JID_USER:
            id->user = pstrdup(id->p, str);
            break;
        case JID_SERVER:
            id->server = pstrdup(id->p, str);
            break;
        case JID_TYPE:
            id->type = pstrdup(id->p, str);
            break;
    }
}

char *jid_full(jid id)
{
    spool s;

    if(id == NULL)
        return NULL;

    /* use cached copy */
    if(id->full != NULL)
        return id->full;

    s = spool_new(id->p);
 
    if(id->type != NULL)
        spooler(s, id->type, ":",s);
    else
        spool_add(s, "jabber:");

    if(id->user != NULL)
        spooler(s, id->user,"@",s);

    spool_add(s, id->server);

    if(id->resource != NULL)
        spooler(s, "/",id->resource,s);

    id->full = spool_print(s);
    return id->full;
}

/* parses a /resource?name=value&foo=bar into an xmlnode representing <resource name="value" foo="bar"/> */
xmlnode jid_xres(jid id)
{
    char *cur, *qmark, *amp, *eq;
    xmlnode x;

    if(id == NULL || id->resource == NULL) return NULL;

    cur = pstrdup(id->p, id->resource);
    qmark = strstr(cur, "?");
    if(qmark == NULL) return NULL;
    *qmark = '\0';
    qmark++;

    x = _xmlnode_new(id->p, cur, NTYPE_TAG);

    cur = qmark;
    while(cur != '\0')
    {
        eq = strstr(cur, "=");
        if(eq == NULL) break;
        *eq = '\0';
        eq++;

        amp = strstr(eq, "&");
        if(amp != NULL)
        {
            *amp = '\0';
            amp++;
        }

        xmlnode_put_attrib(x,cur,eq);

        if(amp != NULL)
            cur = amp;
        else
            break;
    }

    return x;
}

int jid_cmp(jid a, jid b)
{
    if(a == NULL || b == NULL)
        return -1;

    if(a->resource != NULL)
        if(j_strcmp(a->resource, b->resource) != 0) return -1;
    if(a->user != NULL)
        if(j_strcmp(a->user, b->user) != 0) return -1;
    if(a->server != NULL)
        if(j_strcmp(a->server, b->server) != 0) return -1;
    if(a->type != NULL)
        if(j_strcmp(a->type, b->type) != 0) return -1;

    return 0;
}

jid jid_append(jid a, jid b)
{
    jid next;

    if(a == NULL)
        return b;

    next = a;
    while(next != NULL)
    {
        /* check for dups */
        if(jid_cmp(next,b) == 0)
            return a;
        if(next->next == NULL)
            next->next = b;
        next = next->next;
    }
    return a; /* never reached */
}
